@@ -1,4 +1,18 @@
-# -*- test-case-name: buildbot.test.test_svnpoller -*-
+# This file is part of Buildbot.  Buildbot is free software: you can
+# redistribute it and/or modify it under the terms of the GNU General Public
+# License as published by the Free Software Foundation, version 2.
+#
+# This program is distributed in the hope that it will be useful, but WITHOUT
+# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+# details.
+#
+# You should have received a copy of the GNU General Public License along with
+# this program; if not, write to the Free Software Foundation, Inc., 51
+# Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+#
+# Copyright Buildbot Team Members
+
 
 # Based on the work of Dave Peticolas for the P4poll
 # Changed to svn (using xml.dom.minidom) by Niklaus Giger
@@ -13,17 +27,13 @@ from buildbot.changes import base
 from buildbot.changes.changes import Change
 
 import xml.dom.minidom
-import urllib
+import os, urllib
 
 def _assert(condition, msg):
     if condition:
         return True
     raise AssertionError(msg)
 
-def dbgMsg(myString):
-    log.msg(myString)
-    return 1
-
 # these split_file_* functions are available for use as values to the
 # split_file= argument.
 def split_file_alwaystrunk(path):
@@ -48,7 +58,7 @@ class SVNPoller(base.ChangeSource, util.ComparableMixin):
     compare_attrs = ["svnurl", "split_file_function",
                      "svnuser", "svnpasswd",
                      "pollinterval", "histmax",
-                     "svnbin"]
+                     "svnbin", "category", "cachepath"]
 
     parent = None # filled in when we're added
     last_change = None
@@ -58,7 +68,8 @@ class SVNPoller(base.ChangeSource, util.ComparableMixin):
     def __init__(self, svnurl, split_file=None,
                  svnuser=None, svnpasswd=None,
                  pollinterval=10*60, histmax=100,
-                 svnbin='svn', revlinktmpl=''):
+                 svnbin='svn', revlinktmpl='', category=None, 
+                 project='', cachepath=None):
         """
         @type  svnurl: string
         @param svnurl: the SVN URL that describes the repository and
@@ -90,8 +101,8 @@ class SVNPoller(base.ChangeSource, util.ComparableMixin):
                            which is passed to the split_file function.
 
                            If the function returns None, the file is ignored.
-                           Use this to indicate that the file is not a part
-                           of this project.
+                           Use this to indicate that the file is not relevant
+                           to this buildmaster.
                            
                            For example, if your repository puts the trunk in
                            trunk/... and branches are in places like
@@ -123,9 +134,9 @@ class SVNPoller(base.ChangeSource, util.ComparableMixin):
                                 branch = 'branches/' + pieces.pop(0)
                             else:
                                 return None # something weird
-                            projectname = pieces.pop(0)
-                            if projectname != 'ProjectA':
-                                return None # wrong project
+                            productname = pieces.pop(0)
+                            if productname != 'ProjectA':
+                                return None # wrong product
                             return (branch, '/'.join(pieces))
 
                            The default of split_file= is None, which
@@ -164,13 +175,28 @@ class SVNPoller(base.ChangeSource, util.ComparableMixin):
         @param svnbin:       path to svn binary, defaults to just 'svn'. Use
                              this if your subversion command lives in an
                              unusual location.
-	
-	@type  revlinktmpl:  string
-	@param revlinktmpl:  A format string to use for hyperlinks to revision
-			     information. For example, setting this to
-			     "http://reposerver/websvn/revision.php?rev=%s"
-			     would create suitable links on the build pages
-			     to information in websvn on each revision.
+        
+        @type  revlinktmpl:  string
+        @param revlinktmpl:  A format string to use for hyperlinks to revision
+                             information. For example, setting this to
+                             "http://reposerver/websvn/revision.php?rev=%s"
+                             would create suitable links on the build pages
+                             to information in websvn on each revision.
+
+        @type  category:     string
+        @param category:     A single category associated with the changes that
+                             could be used by schedulers watch for branches of a
+                             certain name AND category.
+                             
+        @type  project       string
+        @param project       A single project that the changes are associated with
+                             the repository, added to the changes, for the use in 
+                             change filters
+
+        @type  cachepath     string
+        @param cachepath     A path to a file that can be used to store the last
+                             rev that was processed, so we can grab changes that
+                             happened while we were offline
         """
 
         if svnurl.endswith("/"):
@@ -180,7 +206,10 @@ class SVNPoller(base.ChangeSource, util.ComparableMixin):
         self.svnuser = svnuser
         self.svnpasswd = svnpasswd
 
-	self.revlinktmpl = revlinktmpl
+        self.revlinktmpl = revlinktmpl
+
+        self.environ = os.environ.copy() # include environment variables
+                                         # required for ssh-agent auth
 
         self.svnbin = svnbin
         self.pollinterval = pollinterval
@@ -188,6 +217,20 @@ class SVNPoller(base.ChangeSource, util.ComparableMixin):
         self._prefix = None
         self.overrun_counter = 0
         self.loop = LoopingCall(self.checksvn)
+        self.category = category
+        self.project = project
+
+        self.cachepath = cachepath
+        if self.cachepath and os.path.exists(self.cachepath):
+            try:
+                f = open(self.cachepath, "r")
+                self.last_change = int(f.read().strip())
+                log.msg("SVNPoller(%s) setting last_change to %s" % (self.svnurl, self.last_change))
+                f.close()
+            except:
+                self.cachepath = None
+                log.msg("SVNPoller(%s) cache file corrupt, skipping and not using" % self.svnurl)
+                log.err()
 
     def split_file(self, path):
         # use getattr() to avoid turning this function into a bound method,
@@ -250,7 +293,10 @@ class SVNPoller(base.ChangeSource, util.ComparableMixin):
             return defer.succeed(None)
         self.working = True
 
-        log.msg("SVNPoller polling")
+        if self.project:
+            log.msg("SVNPoller polling " + self.project)
+        else:
+            log.msg("SVNPoller polling")
         if not self._prefix:
             # this sets self._prefix when it finishes. It fires with
             # self._prefix as well, because that makes the unit tests easier
@@ -270,7 +316,7 @@ class SVNPoller(base.ChangeSource, util.ComparableMixin):
 
     def getProcessOutput(self, args):
         # this exists so we can override it during the unit tests
-        d = utils.getProcessOutput(self.svnbin, args, {})
+        d = utils.getProcessOutput(self.svnbin, args, self.environ)
         return d
 
     def get_root(self):
@@ -286,7 +332,6 @@ class SVNPoller(base.ChangeSource, util.ComparableMixin):
         try:
             doc = xml.dom.minidom.parseString(output)
         except xml.parsers.expat.ExpatError:
-            dbgMsg("_process_changes: ExpatError in %s" % output)
             log.msg("SVNPoller._determine_prefix_2: ExpatError in '%s'"
                     % output)
             raise
@@ -325,8 +370,7 @@ class SVNPoller(base.ChangeSource, util.ComparableMixin):
         try:
             doc = xml.dom.minidom.parseString(output)
         except xml.parsers.expat.ExpatError:
-            dbgMsg("_process_changes: ExpatError in %s" % output)
-            log.msg("SVNPoller._parse_changes: ExpatError in '%s'" % output)
+            log.msg("SVNPoller.parse_logs: ExpatError in '%s'" % output)
             raise
         logentries = doc.getElementsByTagName("logentry")
         return logentries
@@ -397,16 +441,15 @@ class SVNPoller(base.ChangeSource, util.ComparableMixin):
         changes = []
 
         for el in new_logentries:
-            branch_files = [] # get oldest change first
             revision = str(el.getAttribute("revision"))
 
-	    revlink=''
+            revlink=''
 
-	    if self.revlinktmpl:
-		if revision:
-		    revlink = self.revlinktmpl % urllib.quote_plus(revision)
+            if self.revlinktmpl:
+                if revision:
+                    revlink = self.revlinktmpl % urllib.quote_plus(revision)
 
-            dbgMsg("Adding change revision %s" % (revision,))
+            log.msg("Adding change revision %s" % (revision,))
             # TODO: the rest of buildbot may not be ready for unicode 'who'
             # values
             author   = self._get_text(el, "author")
@@ -420,7 +463,12 @@ class SVNPoller(base.ChangeSource, util.ComparableMixin):
             #when     = time.mktime(time.strptime("%.19s" % when,
             #                                     "%Y-%m-%dT%H:%M:%S"))
             branches = {}
-            pathlist = el.getElementsByTagName("paths")[0]
+            try:
+                pathlist = el.getElementsByTagName("paths")[0]
+            except IndexError: # weird, we got an empty revision
+                log.msg("ignoring commit with no paths")
+                continue
+
             for p in pathlist.getElementsByTagName("path"):
                 action = p.getAttribute("action")
                 path = "".join([t.data for t in p.childNodes])
@@ -457,7 +505,10 @@ class SVNPoller(base.ChangeSource, util.ComparableMixin):
                                comments=comments,
                                revision=revision,
                                branch=branch,
-			       revlink=revlink)
+                               revlink=revlink,
+                               category=self.category,
+                               repository=self.svnurl,
+                               project = self.project)
                     changes.append(c)
 
         return changes
@@ -467,15 +518,18 @@ class SVNPoller(base.ChangeSource, util.ComparableMixin):
             self.parent.addChange(c)
 
     def finished_ok(self, res):
-        log.msg("SVNPoller finished polling")
-        dbgMsg('_finished : %s' % res)
+        if self.cachepath:
+            f = open(self.cachepath, "w")
+            f.write(str(self.last_change))
+            f.close()
+
+        log.msg("SVNPoller finished polling %s" % res)
         assert self.working
         self.working = False
         return res
 
     def finished_failure(self, f):
-        log.msg("SVNPoller failed")
-        dbgMsg('_finished : %s' % f)
+        log.msg("SVNPoller failed %s" % f)
         assert self.working
         self.working = False
         return None # eat the failure
