@@ -1,3 +1,18 @@
+# This file is part of Buildbot.  Buildbot is free software: you can
+# redistribute it and/or modify it under the terms of the GNU General Public
+# License as published by the Free Software Foundation, version 2.
+#
+# This program is distributed in the hope that it will be useful, but WITHOUT
+# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+# details.
+#
+# You should have received a copy of the GNU General Public License along with
+# this program; if not, write to the Free Software Foundation, Inc., 51
+# Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+#
+# Copyright Buildbot Team Members
+
 import sys
 import os
 from shutil import copy, rmtree
@@ -30,8 +45,8 @@ class ConfigLoader(master.BuildMaster):
             configFile = open(configFileName, "r")
             self.loadConfig(configFile, check_synchronously_only=True)
         except:
+            # clean up before passing on the exception
             os.chdir(dir)
-            configFile.close()
             rmtree(tempdir)
             raise
         os.chdir(dir)
