@@ -1,4 +1,18 @@
-# -*- test-case-name: buildbot.test.test_status -*-
+# This file is part of Buildbot.  Buildbot is free software: you can
+# redistribute it and/or modify it under the terms of the GNU General Public
+# License as published by the Free Software Foundation, version 2.
+#
+# This program is distributed in the hope that it will be useful, but WITHOUT
+# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+# details.
+#
+# You should have received a copy of the GNU General Public License along with
+# this program; if not, write to the Free Software Foundation, Inc., 51
+# Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+#
+# Copyright Buildbot Team Members
+
 
 from twisted.spread import pb
 from twisted.python import components, log as twlog
@@ -40,8 +54,9 @@ class RemoteBuildSet(pb.Referenceable):
 
     def remote_getBuildRequests(self):
         """Returns a list of (builderName, BuildRequest) tuples."""
-        return [(br.getBuilderName(), IRemote(br))
-                for br in self.b.getBuildRequests()]
+        return [(bname, IRemote(br))
+                for (bname, br)
+                in self.b.getBuilderNamesAndBuildRequests().items()]
 
     def remote_isFinished(self):
         return self.b.isFinished()
@@ -70,6 +85,9 @@ class RemoteBuilder(pb.Referenceable):
     def remote_getName(self):
         return self.b.getName()
 
+    def remote_getCategory(self):
+        return self.b.getCategory()
+
     def remote_getState(self):
         state, builds = self.b.getState()
         return (state,
