@@ -1,33 +1,48 @@
-# -*- test-case-name: buildbot.test.test_status -*-
+# This file is part of Buildbot.  Buildbot is free software: you can
+# redistribute it and/or modify it under the terms of the GNU General Public
+# License as published by the Free Software Foundation, version 2.
+#
+# This program is distributed in the hope that it will be useful, but WITHOUT
+# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+# details.
+#
+# You should have received a copy of the GNU General Public License along with
+# this program; if not, write to the Free Software Foundation, Inc., 51
+# Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+#
+# Copyright Buildbot Team Members
+
 
-# the email.MIMEMultipart module is only available in python-2.2.2 and later
 import re
 
 from email.Message import Message
 from email.Utils import formatdate
 from email.MIMEText import MIMEText
-try:
-    from email.MIMEMultipart import MIMEMultipart
-    canDoAttachments = True
-except ImportError:
-    canDoAttachments = False
+from email.MIMEMultipart import MIMEMultipart
+from StringIO import StringIO
 import urllib
 
 from zope.interface import implements
-from twisted.internet import defer
-from twisted.mail.smtp import sendmail
+from twisted.internet import defer, reactor
+from twisted.mail.smtp import ESMTPSenderFactory
 from twisted.python import log as twlog
 
+have_ssl = True
+try:
+    from twisted.internet import ssl
+    from OpenSSL.SSL import SSLv3_METHOD
+except ImportError:
+    have_ssl = False
+
 from buildbot import interfaces, util
 from buildbot.status import base
-from buildbot.status.builder import FAILURE, SUCCESS, WARNINGS, Results
-
-import sys
-if sys.version_info[:3] < (2,4,0):
-    from sets import Set as set
+from buildbot.status.builder import FAILURE, SUCCESS, Results
 
 VALID_EMAIL = re.compile("[a-zA-Z0-9\.\_\%\-\+]+@[a-zA-Z0-9\.\_\%\-]+.[a-zA-Z]{2,6}")
 
+ENCODING = 'utf8'
+
 class Domain(util.ComparableMixin):
     implements(interfaces.IEmailLookup)
     compare_attrs = ["domain"]
@@ -43,6 +58,75 @@ class Domain(util.ComparableMixin):
         return name + "@" + self.domain
 
 
+def defaultMessage(mode, name, build, results, master_status):
+    """Generate a buildbot mail message and return a tuple of message text
+        and type."""
+    result = Results[results]
+    ss = build.getSourceStamp()
+
+    text = ""
+    if mode == "all":
+        text += "The Buildbot has finished a build"
+    elif mode == "failing":
+        text += "The Buildbot has detected a failed build"
+    elif mode == "warnings":
+        text += "The Buildbot has detected a problem in the build"
+    elif mode == "passing":
+        text += "The Buildbot has detected a passing build"
+    elif mode == "change" and result == 'success':
+        text += "The Buildbot has detected a restored build"
+    else:    
+        text += "The Buildbot has detected a new failure"
+    if ss and ss.project:
+        project = ss.project
+    else:
+        project = master_status.getProjectName()
+    text += " on builder %s while building %s.\n" % (name, project)
+    if master_status.getURLForThing(build):
+        text += "Full details are available at:\n %s\n" % master_status.getURLForThing(build)
+    text += "\n"
+
+    if master_status.getBuildbotURL():
+        text += "Buildbot URL: %s\n\n" % urllib.quote(master_status.getBuildbotURL(), '/:')
+
+    text += "Buildslave for this Build: %s\n\n" % build.getSlavename()
+    text += "Build Reason: %s\n" % build.getReason()
+
+    source = ""
+    if ss and ss.branch:
+        source += "[branch %s] " % ss.branch
+    if ss and ss.revision:
+        source += str(ss.revision)
+    else:
+        source += "HEAD"
+    if ss and ss.patch:
+        source += " (plus patch)"
+
+    text += "Build Source Stamp: %s\n" % source
+
+    text += "Blamelist: %s\n" % ",".join(build.getResponsibleUsers())
+
+    text += "\n"
+
+    t = build.getText()
+    if t:
+        t = ": " + " ".join(t)
+    else:
+        t = ""
+
+    if result == 'success':
+        text += "Build succeeded!\n"
+    elif result == 'warnings':
+        text += "Build Had Warnings%s\n" % t
+    else:
+        text += "BUILD FAILED%s\n" % t
+
+    text += "\n"
+    text += "sincerely,\n"
+    text += " -The Buildbot\n"
+    text += "\n"
+    return { 'body' : text, 'type' : 'plain' }
+
 class MailNotifier(base.StatusReceiverMultiService):
     """This is a status notifier which sends email to a list of recipients
     upon the completion of each build. It can be configured to only send out
@@ -74,12 +158,14 @@ class MailNotifier(base.StatusReceiverMultiService):
                  subject="buildbot %(result)s in %(projectName)s on %(builder)s",
                  lookup=None, extraRecipients=[],
                  sendToInterestedUsers=True, customMesg=None,
-                 messageFormatter=None, extraHeaders=None, addPatch=True):
+                 messageFormatter=defaultMessage, extraHeaders=None,
+                 addPatch=True, useTls=False, 
+                 smtpUser=None, smtpPassword=None, smtpPort=25):
         """
         @type  fromaddr: string
         @param fromaddr: the email address to be used in the 'From' header.
         @type  sendToInterestedUsers: boolean
-        @param sendToInterestedUsers: if True (the default), send mail to all 
+        @param sendToInterestedUsers: if True (the default), send mail to all
                                       of the Interested Users. If False, only
                                       send mail to the extraRecipients list.
 
@@ -90,7 +176,9 @@ class MailNotifier(base.StatusReceiverMultiService):
                                 developers who made Changes that went into this
                                 build). It is a good idea to create a small
                                 mailing list and deliver to that, then let
-                                subscribers come and go as they please.
+                                subscribers come and go as they please.  The
+                                addresses in this list are used literally (they
+                                are not processed by lookup).
 
         @type  subject: string
         @param subject: a string to be used as the subject line of the message.
@@ -101,6 +189,7 @@ class MailNotifier(base.StatusReceiverMultiService):
         @param mode: one of:
                      - 'all': send mail about all builds, passing and failing
                      - 'failing': only send mail about builds which fail
+                     - 'warnings': send mail if builds contain warnings or fail 
                      - 'passing': only send mail about builds which succeed
                      - 'problem': only send mail about a build which failed
                      when the previous build passed
@@ -133,23 +222,23 @@ class MailNotifier(base.StatusReceiverMultiService):
 
         @type  lookup:    implementor of {IEmailLookup}
         @param lookup:    object which provides IEmailLookup, which is
-                          responsible for mapping User names (which come from
-                          the VC system) into valid email addresses. If not
-                          provided, the notifier will only be able to send mail
-                          to the addresses in the extraRecipients list. Most of
-                          the time you can use a simple Domain instance. As a
-                          shortcut, you can pass as string: this will be
-                          treated as if you had provided Domain(str). For
-                          example, lookup='twistedmatrix.com' will allow mail
-                          to be sent to all developers whose SVN usernames
-                          match their twistedmatrix.com account names.
-                          
+                          responsible for mapping User names for Interested
+                          Users (which come from the VC system) into valid
+                          email addresses. If not provided, the notifier will
+                          only be able to send mail to the addresses in the
+                          extraRecipients list. Most of the time you can use a
+                          simple Domain instance. As a shortcut, you can pass
+                          as string: this will be treated as if you had provided
+                          Domain(str). For example, lookup='twistedmatrix.com'
+                          will allow mail to be sent to all developers whose SVN
+                          usernames match their twistedmatrix.com account names.
+
         @type  customMesg: func
         @param customMesg: (this function is deprecated)
 
         @type  messageFormatter: func
         @param messageFormatter: function taking (mode, name, build, result,
-                                 master_status ) and returning a dictionary
+                                 master_status) and returning a dictionary
                                  containing two required keys "body" and "type",
                                  with a third optional key, "subject". The
                                  "body" key gives a string that contains the
@@ -164,17 +253,34 @@ class MailNotifier(base.StatusReceiverMultiService):
                              best to avoid putting 'To', 'From', 'Date',
                              'Subject', or 'CC' in here. Both the names and
                              values may be WithProperties instances.
+
+        @type useTls: boolean
+        @param useTls: Send emails using TLS and authenticate with the 
+                       smtp host. Defaults to False.
+
+        @type smtpUser: string
+        @param smtpUser: The user that will attempt to authenticate with the
+                         relayhost when useTls is True.
+
+        @type smtpPassword: string
+        @param smtpPassword: The password that smtpUser will use when
+                             authenticating with relayhost.
+
+        @type smtpPort: int
+        @param smtpPort: The port that will be used when connecting to the
+                         relayhost. Defaults to 25.
         """
 
         base.StatusReceiverMultiService.__init__(self)
         assert isinstance(extraRecipients, (list, tuple))
         for r in extraRecipients:
             assert isinstance(r, str)
-            assert VALID_EMAIL.search(r) # require full email addresses, not User names
+            # require full email addresses, not User names
+            assert VALID_EMAIL.search(r), "%s is not a valid email" % r 
         self.extraRecipients = extraRecipients
         self.sendToInterestedUsers = sendToInterestedUsers
         self.fromaddr = fromaddr
-        assert mode in ('all', 'failing', 'problem', 'change', 'passing')
+        assert mode in ('all', 'failing', 'problem', 'change', 'passing', 'warnings')
         self.mode = mode
         self.categories = categories
         self.builders = builders
@@ -192,17 +298,17 @@ class MailNotifier(base.StatusReceiverMultiService):
             assert isinstance(extraHeaders, dict)
         self.extraHeaders = extraHeaders
         self.addPatch = addPatch
+        self.useTls = useTls
+        self.smtpUser = smtpUser
+        self.smtpPassword = smtpPassword
+        self.smtpPort = smtpPort
         self.watched = []
         self.master_status = None
 
         # you should either limit on builders or categories, not both
         if self.builders != None and self.categories != None:
-            twlog.err("Please specify only builders to ignore or categories to include")
-            raise # FIXME: the asserts above do not raise some Exception either
-
-        if customMesg and messageFormatter:
-            twlog.err("Specify only one of customMesg and messageFormatter")
-            self.customMesg = None
+            twlog.err("Please specify only builders or categories to include not both.")
+            raise interfaces.ParameterError("Please specify only builders or categories to include not both.")
 
         if customMesg:
             twlog.msg("customMesg is deprecated; please use messageFormatter instead")
@@ -248,6 +354,8 @@ class MailNotifier(base.StatusReceiverMultiService):
                builder.category not in self.categories:
             return # ignore this build
 
+        if self.mode == "warnings" and results == SUCCESS:
+            return
         if self.mode == "failing" and results != FAILURE:
             return
         if self.mode == "passing" and results != SUCCESS:
@@ -261,10 +369,6 @@ class MailNotifier(base.StatusReceiverMultiService):
         if self.mode == "change":
             prev = build.getPreviousBuild()
             if not prev or prev.getResults() == results:
-                if prev:
-                    print prev.getResults()
-                else:
-                    print "no prev"
                 return
         # for testing purposes, buildMessage returns a Deferred that fires
         # when the mail has been sent. To help unit tests, we return that
@@ -290,8 +394,6 @@ class MailNotifier(base.StatusReceiverMultiService):
                          logf.getText().splitlines(),
                          logStatus))
 
-        properties = build.getProperties()
-                
         attrs = {'builderName': name,
                  'projectName': master_status.getProjectName(),
                  'mode': mode,
@@ -299,7 +401,7 @@ class MailNotifier(base.StatusReceiverMultiService):
                  'buildURL': master_status.getURLForThing(build),
                  'buildbotURL': master_status.getBuildbotURL(),
                  'buildText': build.getText(),
-                 'buildProperties': properties,
+                 'buildProperties': build.getProperties(),
                  'slavename': build.getSlavename(),
                  'reason':  build.getReason(),
                  'responsibleUsers': build.getResponsibleUsers(),
@@ -318,109 +420,27 @@ class MailNotifier(base.StatusReceiverMultiService):
 
         return attrs
 
-    def defaultMessage(self, mode, name, build, results, master_status):
-        """Generate a buildbot mail message and return a tuple of message text
-        and type."""
-        result = Results[results]
-
-        text = ""
-        if mode == "all":
-            text += "The Buildbot has finished a build"
-        elif mode == "failing":
-            text += "The Buildbot has detected a failed build"
-        elif mode == "passing":
-            text += "The Buildbot has detected a passing build"
-        elif mode == "change" and result == 'success':
-            text += "The Buildbot has detected a restored build"
-        else:    
-            text += "The Buildbot has detected a new failure"
-        text += " of %s on %s.\n" % (name, master_status.getProjectName())
-        if master_status.getURLForThing(build):
-            text += "Full details are available at:\n %s\n" % master_status.getURLForThing(build)
-        text += "\n"
-
-        if master_status.getBuildbotURL():
-            text += "Buildbot URL: %s\n\n" % urllib.quote(master_status.getBuildbotURL(), '/:')
-
-        text += "Buildslave for this Build: %s\n\n" % build.getSlavename()
-        text += "Build Reason: %s\n" % build.getReason()
-
-        source = ""
-        ss = build.getSourceStamp()
-        if ss and ss.branch:
-            source += "[branch %s] " % ss.branch
-        if ss and ss.revision:
-            source += str(ss.revision)
-        else:
-            source += "HEAD"
-        if ss and ss.patch:
-            source += " (plus patch)"
-
-        text += "Build Source Stamp: %s\n" % source
-
-        text += "Blamelist: %s\n" % ",".join(build.getResponsibleUsers())
-
-        text += "\n"
-
-        t = build.getText()
-        if t:
-            t = ": " + " ".join(t)
-        else:
-            t = ""
-
-        if result == 'success':
-            text += "Build succeeded!\n"
-        elif result == 'warnings':
-            text += "Build Had Warnings%s\n" % t
-        else:
-            text += "BUILD FAILED%s\n" % t
-
-        text += "\n"
-        text += "sincerely,\n"
-        text += " -The Buildbot\n"
-        text += "\n"
-        return { 'body' : text, 'type' : 'plain' }
-
-    def buildMessage(self, name, build, results):
-        if self.customMesg:
-            # the customMesg stuff can be *huge*, so we prefer not to load it
-            attrs = self.getCustomMesgData(self.mode, name, build, results, self.master_status)
-            text, type = self.customMesg(attrs)
-            msgdict = { 'body' : text, 'type' : type }
-        elif self.messageFormatter:
-            msgdict = self.messageFormatter(self.mode, name, build, results, self.master_status)
-        else:
-            msgdict = self.defaultMessage(self.mode, name, build, results, self.master_status)
-
-        text = msgdict['body']
+    def createEmail(self, msgdict, builderName, projectName, results, build,
+                    patch=None, logs=None):
+        text = msgdict['body'].encode(ENCODING)
         type = msgdict['type']
         if 'subject' in msgdict:
-            subject = msgdict['subject']
+            subject = msgdict['subject'].encode(ENCODING)
         else:
             subject = self.subject % { 'result': Results[results],
-                                       'projectName': self.master_status.getProjectName(),
-                                       'builder': name,
+                                       'projectName': projectName,
+                                       'builder': builderName,
                                        }
 
 
         assert type in ('plain', 'html'), "'%s' message type must be 'plain' or 'html'." % type
 
-        haveAttachments = False
-        ss = build.getSourceStamp()
-        if (ss and ss.patch and self.addPatch) or self.addLogs:
-            haveAttachments = True
-            if not canDoAttachments:
-                twlog.msg("warning: I want to send mail with attachments, "
-                          "but this python is too old to have "
-                          "email.MIMEMultipart . Please upgrade to python-2.3 "
-                          "or newer to enable addLogs=True")
-
-        if haveAttachments and canDoAttachments:
+        if patch or logs:
             m = MIMEMultipart()
-            m.attach(MIMEText(text, type))
+            m.attach(MIMEText(text, type, ENCODING))
         else:
             m = Message()
-            m.set_payload(text)
+            m.set_payload(text, ENCODING)
             m.set_type("text/%s" % type)
 
         m['Date'] = formatdate(localtime=True)
@@ -428,18 +448,18 @@ class MailNotifier(base.StatusReceiverMultiService):
         m['From'] = self.fromaddr
         # m['To'] is added later
 
-        if ss and ss.patch and self.addPatch:
-            patch = ss.patch
-            a = MIMEText(patch[1])
+        if patch:
+            a = MIMEText(patch[1].encode(ENCODING), _charset=ENCODING)
             a.add_header('Content-Disposition', "attachment",
                          filename="source patch")
             m.attach(a)
-        if self.addLogs:
-            for log in build.getLogs():
+        if logs:
+            for log in logs:
                 name = "%s.%s" % (log.getStep().getName(),
                                   log.getName())
                 if self._shouldAttachLog(log.getName()) or self._shouldAttachLog(name):
-                    a = MIMEText(log.getText())
+                    a = MIMEText(log.getText().encode(ENCODING), 
+                                 _charset=ENCODING)
                     a.add_header('Content-Disposition', "attachment",
                                  filename=name)
                     m.attach(a)
@@ -447,15 +467,38 @@ class MailNotifier(base.StatusReceiverMultiService):
         # Add any extra headers that were requested, doing WithProperties
         # interpolation if necessary
         if self.extraHeaders:
+            properties = build.getProperties()
             for k,v in self.extraHeaders.items():
                 k = properties.render(k)
                 if k in m:
-                    twlog("Warning: Got header " + k + " in self.extraHeaders "
+                    twlog.msg("Warning: Got header " + k + " in self.extraHeaders "
                           "but it already exists in the Message - "
                           "not adding it.")
                     continue
                 m[k] = properties.render(v)
 
+        return m
+
+    def buildMessage(self, name, build, results):
+        if self.customMesg:
+            # the customMesg stuff can be *huge*, so we prefer not to load it
+            attrs = self.getCustomMesgData(self.mode, name, build, results, self.master_status)
+            text, type = self.customMesg(attrs)
+            msgdict = { 'body' : text, 'type' : type }
+        else:
+            msgdict = self.messageFormatter(self.mode, name, build, results, self.master_status)
+
+        patch = None
+        ss = build.getSourceStamp()
+        if ss and ss.patch and self.addPatch:
+            patch == ss.patch
+        logs = None
+        if self.addLogs:
+            logs = build.getLogs()
+            twlog.err("LOG: %s" % str(logs))
+        m = self.createEmail(msgdict, name, self.master_status.getProjectName(),
+                             results, build, patch, logs)
+
         # now, who is this message going to?
         dl = []
         recipients = []
@@ -474,7 +517,8 @@ class MailNotifier(base.StatusReceiverMultiService):
         return logname in self.addLogs
 
     def _gotRecipients(self, res, rlist, m):
-        recipients = set()
+        to_recipients = set()
+        cc_recipients = set()
 
         for r in rlist:
             if r is None: # getAddress didn't like this address
@@ -486,32 +530,51 @@ class MailNotifier(base.StatusReceiverMultiService):
                 r = r[:r.rindex('@')]
 
             if VALID_EMAIL.search(r):
-                recipients.add(r)
+                to_recipients.add(r)
             else:
                 twlog.msg("INVALID EMAIL: %r" + r)
 
-        # if we're sending to interested users move the extra's to the CC
-        # list so they can tell if they are also interested in the change
-        # unless there are no interested users
-        if self.sendToInterestedUsers and len(recipients):
-            extra_recips = self.extraRecipients[:]
-            extra_recips.sort()
-            m['CC'] = ", ".join(extra_recips)
+        # If we're sending to interested users put the extras in the
+        # CC list so they can tell if they are also interested in the
+        # change:
+        if self.sendToInterestedUsers and to_recipients:
+            cc_recipients.update(self.extraRecipients)
         else:
-            [recipients.add(r) for r in self.extraRecipients[:]]
+            to_recipients.update(self.extraRecipients)
 
-        rlist = list(recipients)
-        rlist.sort()
-        m['To'] = ", ".join(rlist)
+        m['To'] = ", ".join(sorted(to_recipients))
+        if cc_recipients:
+            m['CC'] = ", ".join(sorted(cc_recipients))
 
-        # The extras weren't part of the TO list so add them now
-        if self.sendToInterestedUsers:
-            for r in self.extraRecipients:
-                recipients.add(r)
+        return self.sendMessage(m, list(to_recipients | cc_recipients))
 
-        return self.sendMessage(m, list(recipients))
+    def sendmail(self, s, recipients):
+        result = defer.Deferred()
+        
+        if have_ssl and self.useTls:
+            client_factory = ssl.ClientContextFactory()
+            client_factory.method = SSLv3_METHOD
+        else:
+            client_factory = None
+
+        if self.smtpUser and self.smtpPassword:
+            useAuth = True
+        else:
+            useAuth = False
+        
+        sender_factory = ESMTPSenderFactory(
+            self.smtpUser, self.smtpPassword,
+            self.fromaddr, recipients, StringIO(s),
+            result, contextFactory=client_factory,
+            requireTransportSecurity=self.useTls,
+            requireAuthentication=useAuth)
+
+        reactor.connectTCP(self.relayhost, self.smtpPort, sender_factory)
+        
+        return result
 
     def sendMessage(self, m, recipients):
         s = m.as_string()
         twlog.msg("sending mail (%d bytes) to" % len(s), recipients)
-        return sendmail(self.relayhost, self.fromaddr, recipients, s)
+        return self.sendmail(s, recipients)
+
