@@ -1,21 +1,50 @@
-# -*- test-case-name: buildbot.test.test_web -*-
+# This file is part of Buildbot.  Buildbot is free software: you can
+# redistribute it and/or modify it under the terms of the GNU General Public
+# License as published by the Free Software Foundation, version 2.
+#
+# This program is distributed in the hope that it will be useful, but WITHOUT
+# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+# details.
+#
+# You should have received a copy of the GNU General Public License along with
+# this program; if not, write to the Free Software Foundation, Inc., 51
+# Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+#
+# Copyright Buildbot Team Members
+
 
 from zope.interface import implements
 from twisted.python import log, components
-from twisted.web import html
 import urllib
 
-import time
+import time, locale
 import operator
 
 from buildbot import interfaces, util
-from buildbot import version
 from buildbot.status import builder
 
 from buildbot.status.web.base import Box, HtmlResource, IBox, ICurrentBox, \
-     ITopBox, td, build_get_class, path_to_build, path_to_step, map_branches
+     ITopBox, build_get_class, path_to_build, path_to_step, path_to_root, \
+     map_branches
 
 
+def earlier(old, new):
+    # minimum of two things, but "None" counts as +infinity
+    if old:
+        if new < old:
+            return new
+        return old
+    return new
+
+def later(old, new):
+    # maximum of two things, but "None" counts as -infinity
+    if old:
+        if new > old:
+            return new
+        return old
+    return new
+
 
 class CurrentBox(components.Adapter):
     # this provides the "current activity" box, just above the builder name
@@ -101,16 +130,13 @@ class BuildTopBox(components.Adapter):
         if not builds:
             return Box(["none"], class_="LastBuild")
         b = builds[0]
-        name = b.getBuilder().getName()
-        number = b.getNumber()
         url = path_to_build(req, b)
         text = b.getText()
         tests_failed = b.getSummaryStatistic('tests-failed', operator.add, 0)
         if tests_failed: text.extend(["Failed tests: %d" % tests_failed])
         # TODO: maybe add logs?
-        # TODO: add link to the per-build page at 'url'
         class_ = build_get_class(b)
-        return Box(text, class_="LastBuild %s" % class_)
+        return Box(text, urlbase=url, class_="LastBuild %s" % class_)
 components.registerAdapter(BuildTopBox, builder.BuilderStatus, ITopBox)
 
 class BuildBox(components.Adapter):
@@ -122,8 +148,8 @@ class BuildBox(components.Adapter):
         number = b.getNumber()
         url = path_to_build(req, b)
         reason = b.getReason()
-        text = ('<a title="Reason: %s" href="%s">Build %d</a>'
-                % (html.escape(reason), url, number))
+        template = req.site.buildbot_service.templates.get_template("box_macros.html")
+        text = template.module.build_box(reason=reason,url=url,number=number)
         class_ = "start"
         if b.isFinished() and not b.getSteps():
             # the steps have been pruned, so there won't be any indication
@@ -143,18 +169,23 @@ class StepBox(components.Adapter):
             text = []
         text = text[:]
         logs = self.original.getLogs()
+        
+        cxt = dict(text=text, logs=[], urls=[])
+
         for num in range(len(logs)):
             name = logs[num].getName()
             if logs[num].hasContents():
                 url = urlbase + "/logs/%s" % urllib.quote(name)
-                text.append("<a href=\"%s\">%s</a>" % (url, html.escape(name)))
             else:
-                text.append(html.escape(name))
-        urls = self.original.getURLs()
-        ex_url_class = "BuildStep external"
-        for name, target in urls.items():
-            text.append('[<a href="%s" class="%s">%s</a>]' %
-                        (target, ex_url_class, html.escape(name)))
+                url = None
+            cxt['logs'].append(dict(name=name, url=url))
+
+        for name, target in self.original.getURLs().items():
+            cxt['urls'].append(dict(link=target,name=name))
+
+        template = req.site.buildbot_service.templates.get_template("box_macros.html")
+        text = template.module.step_box(**cxt)
+        
         class_ = "BuildStep " + build_get_class(self.original)
         return Box(text, class_=class_)
 components.registerAdapter(StepBox, builder.BuildStepStatus, IBox)
@@ -230,91 +261,6 @@ def insertGaps(g, showEvents, lastEventTime, idleGap=2):
         followingEventStarts = starts
         if debug: log.msg(" fES1", starts)
 
-HELP = '''
-<form action="../waterfall" method="GET">
-
-<h1>The Waterfall Display</h1>
-
-<p>The Waterfall display can be controlled by adding query arguments to the
-URL. For example, if your Waterfall is accessed via the URL
-<tt>http://buildbot.example.org:8080</tt>, then you could add a
-<tt>branch=</tt> argument (described below) by going to
-<tt>http://buildbot.example.org:8080?branch=beta4</tt> instead. Remember that
-query arguments are separated from each other with ampersands, but they are
-separated from the main URL with a question mark, so to add a
-<tt>branch=</tt> and two <tt>builder=</tt> arguments, you would use
-<tt>http://buildbot.example.org:8080?branch=beta4&amp;builder=unix&amp;builder=macos</tt>.</p>
-
-<h2>Limiting the Displayed Interval</h2>
-
-<p>The <tt>last_time=</tt> argument is a unix timestamp (seconds since the
-start of 1970) that will be used as an upper bound on the interval of events
-displayed: nothing will be shown that is more recent than the given time.
-When no argument is provided, all events up to and including the most recent
-steps are included.</p>
-
-<p>The <tt>first_time=</tt> argument provides the lower bound. No events will
-be displayed that occurred <b>before</b> this timestamp. Instead of providing
-<tt>first_time=</tt>, you can provide <tt>show_time=</tt>: in this case,
-<tt>first_time</tt> will be set equal to <tt>last_time</tt> minus
-<tt>show_time</tt>. <tt>show_time</tt> overrides <tt>first_time</tt>.</p>
-
-<p>The display normally shows the latest 200 events that occurred in the
-given interval, where each timestamp on the left hand edge counts as a single
-event. You can add a <tt>num_events=</tt> argument to override this this.</p>
-
-<h2>Showing non-Build events</h2>
-
-<p>By passing <tt>show_events=true</tt>, you can add the "buildslave
-attached", "buildslave detached", and "builder reconfigured" events that
-appear in-between the actual builds.</p>
-
-%(show_events_input)s
-
-<h2>Showing only the Builders with failures</h2>
-
-<p>By adding the <tt>failures_only=true</tt> argument, the display will be limited
-to showing builders that are currently failing. A builder is considered
-failing if the last finished build was not successful, a step in the current
-build(s) failed, or if the builder is offline.
-
-%(failures_only_input)s
-
-<h2>Showing only Certain Branches</h2>
-
-<p>If you provide one or more <tt>branch=</tt> arguments, the display will be
-limited to builds that used one of the given branches. If no <tt>branch=</tt>
-arguments are given, builds from all branches will be displayed.</p>
-
-Erase the text from these "Show Branch:" boxes to remove that branch filter.
-
-%(show_branches_input)s
-
-<h2>Limiting the Builders that are Displayed</h2>
-
-<p>By adding one or more <tt>builder=</tt> arguments, the display will be
-limited to showing builds that ran on the given builders. This serves to
-limit the display to the specific named columns. If no <tt>builder=</tt>
-arguments are provided, all Builders will be displayed.</p>
-
-<p>To view a Waterfall page with only a subset of Builders displayed, select
-the Builders you are interested in here.</p>
-
-%(show_builders_input)s
-
-
-<h2>Auto-reloading the Page</h2>
-
-<p>Adding a <tt>reload=</tt> argument will cause the page to automatically
-reload itself after that many seconds.</p>
-
-%(show_reload_input)s
-
-<h2>Reload Waterfall Page</h2>
-
-<input type="submit" value="View Waterfall" />
-</form>
-'''
 
 class WaterfallHelp(HtmlResource):
     title = "Waterfall Help"
@@ -323,66 +269,23 @@ class WaterfallHelp(HtmlResource):
         HtmlResource.__init__(self)
         self.categories = categories
 
-    def body(self, request):
-        data = ''
+    def content(self, request, cxt):
         status = self.getStatus(request)
 
-        showEvents_checked = ''
-        if request.args.get("show_events", ["false"])[0].lower() == "true":
-            showEvents_checked = 'checked="checked"'
-        show_events_input = ('<p>'
-                             '<input type="checkbox" name="show_events" '
-                             'value="true" %s>'
-                             'Show non-Build events'
-                             '</p>\n'
-                             ) % showEvents_checked
-
-        failuresOnly_checked = ''
-        if request.args.get("failures_only", ["false"])[0].lower() == "true":
-            failuresOnly_checked = 'checked="checked"'
-        failures_only_input = ('<p>'
-                               '<input type="checkbox" name="failures_only" '
-                               'value="true" %s>'
-                               'Show failures only'
-                               '</p>\n'
-                               ) % failuresOnly_checked
-
-        branches = [b
-                    for b in request.args.get("branch", [])
-                    if b]
-        branches.append('')
-        show_branches_input = '<table>\n'
-        for b in branches:
-            show_branches_input += ('<tr>'
-                                    '<td>Show Branch: '
-                                    '<input type="text" name="branch" '
-                                    'value="%s">'
-                                    '</td></tr>\n'
-                                    ) % (html.escape(b),)
-        show_branches_input += '</table>\n'
+        cxt['show_events_checked'] = request.args.get("show_events", ["false"])[0].lower() == "true"
+        cxt['branches'] = [b for b in request.args.get("branch", []) if b]
+        cxt['failures_only'] = request.args.get("failures_only", ["false"])[0].lower() == "true"
+        cxt['committers'] = [c for c in request.args.get("committer", []) if c]
 
         # this has a set of toggle-buttons to let the user choose the
         # builders
-        showBuilders = request.args.get("show", [])
-        showBuilders.extend(request.args.get("builder", []))
-        allBuilders = status.getBuilderNames(categories=self.categories)
-
-        show_builders_input = '<table>\n'
-        for bn in allBuilders:
-            checked = ""
-            if bn in showBuilders:
-                checked = 'checked="checked"'
-            show_builders_input += ('<tr>'
-                                    '<td><input type="checkbox"'
-                                    ' name="builder" '
-                                    'value="%s" %s></td> '
-                                    '<td>%s</td></tr>\n'
-                                    ) % (bn, checked, bn)
-        show_builders_input += '</table>\n'
+        show_builders = request.args.get("show", [])
+        show_builders.extend(request.args.get("builder", []))
+        cxt['show_builders'] = show_builders
+        cxt['all_builders'] = status.getBuilderNames(categories=self.categories)
 
         # a couple of radio-button selectors for refresh time will appear
         # just after that text
-        show_reload_input = '<table>\n'
         times = [("none", "None"),
                  ("60", "60 seconds"),
                  ("300", "5 minutes"),
@@ -393,25 +296,13 @@ class WaterfallHelp(HtmlResource):
             current_reload_time = current_reload_time[0]
         if current_reload_time not in [t[0] for t in times]:
             times.insert(0, (current_reload_time, current_reload_time) )
-        for value, name in times:
-            checked = ""
-            if value == current_reload_time:
-                checked = 'checked="checked"'
-            show_reload_input += ('<tr>'
-                                  '<td><input type="radio" name="reload" '
-                                  'value="%s" %s></td> '
-                                  '<td>%s</td></tr>\n'
-                                  ) % (html.escape(value), checked, html.escape(name))
-        show_reload_input += '</table>\n'
 
-        fields = {"show_events_input": show_events_input,
-                  "show_branches_input": show_branches_input,
-                  "show_builders_input": show_builders_input,
-                  "show_reload_input": show_reload_input,
-                  "failures_only_input": failures_only_input,
-                  }
-        data += HELP % fields
-        return data
+        cxt['times'] = times
+        cxt['current_reload_time'] = current_reload_time   
+
+        template = request.site.buildbot_service.templates.get_template("waterfallhelp.html")
+        return template.render(**cxt)
+
 
 class WaterfallStatusResource(HtmlResource):
     """This builds the main status page, with the waterfall display, and
@@ -432,7 +323,7 @@ class WaterfallStatusResource(HtmlResource):
         else:
             return "BuildBot"
 
-    def getChangemaster(self, request):
+    def getChangeManager(self, request):
         # TODO: this wants to go away, access it through IStatus
         return request.site.buildbot_service.getChangeSvc()
 
@@ -445,13 +336,6 @@ class WaterfallStatusResource(HtmlResource):
                 pass
         return None
 
-    def head(self, request):
-        head = ''
-        reload_time = self.get_reload_time(request)
-        if reload_time is not None:
-            head += '<meta http-equiv="refresh" content="%d">\n' % reload_time
-        return head
-
     def isSuccess(self, builderStatus):
         # Helper function to return True if the builder is not failing.
         # The function will return false if the current state is "offline",
@@ -480,17 +364,9 @@ class WaterfallStatusResource(HtmlResource):
         # don't have any failed steps.
         return True
 
-    def body(self, request):
-        "This method builds the main waterfall display."
-
+    def content(self, request, ctx):
         status = self.getStatus(request)
-        data = ''
-
-        projectName = status.getProjectName()
-        projectURL = status.getProjectURL()
-
-        phase = request.args.get("phase",["2"])
-        phase = int(phase[0])
+        ctx['refresh'] = self.get_reload_time(request)
 
         # we start with all Builders available to this Waterfall: this is
         # limited by the config-file -time categories= argument, and defaults
@@ -520,60 +396,35 @@ class WaterfallStatusResource(HtmlResource):
         failuresOnly = request.args.get("failures_only", ["false"])[0]
         if failuresOnly.lower() == "true":
             builders = [b for b in builders if not self.isSuccess(b)]
-
-        builderNames = [b.name for b in builders]
-
-        if phase == -1:
-            return self.body0(request, builders)
-        (changeNames, builderNames, timestamps, eventGrid, sourceEvents) = \
-                      self.buildGrid(request, builders)
-        if phase == 0:
-            return self.phase0(request, (changeNames + builderNames),
-                               timestamps, eventGrid)
-        # start the table: top-header material
-        data += '<table border="0" cellspacing="0">\n'
-
-        if projectName and projectURL:
-            # TODO: this is going to look really ugly
-            topleft = '<a href="%s">%s</a><br />last build' % \
-                      (projectURL, projectName)
-        else:
-            topleft = "last build"
-        data += ' <tr class="LastBuild">\n'
-        data += td(topleft, align="right", colspan=2, class_="Project")
-        for b in builders:
-            box = ITopBox(b).getBox(request)
-            data += box.td(align="center")
-        data += " </tr>\n"
-
-        data += ' <tr class="Activity">\n'
-        data += td('current activity', align='right', colspan=2)
-        for b in builders:
-            box = ICurrentBox(b).getBox(status)
-            data += box.td(align="center")
-        data += " </tr>\n"
         
-        data += " <tr>\n"
-        TZ = time.tzname[time.localtime()[-1]]
-        data += td("time (%s)" % TZ, align="center", class_="Time")
-        data += td('<a href="%s">changes</a>' % request.childLink("../changes"),
-                   align="center", class_="Change")
-        for name in builderNames:
-            safename = urllib.quote(name, safe='')
-            data += td('<a href="%s">%s</a>' %
-                       (request.childLink("../builders/%s" % safename), name),
-                       align="center", class_="Builder")
-        data += " </tr>\n"
-
-        if phase == 1:
-            f = self.phase1
+        (changeNames, builderNames, timestamps, eventGrid, sourceEvents) = \
+                      self.buildGrid(request, builders)            
+            
+        # start the table: top-header material
+        locale_enc = locale.getdefaultlocale()[1]
+        if locale_enc is not None:
+            locale_tz = unicode(time.tzname[time.localtime()[-1]], locale_enc)
         else:
-            f = self.phase2
-        data += f(request, changeNames + builderNames, timestamps, eventGrid,
-                  sourceEvents)
-
-        data += "</table>\n"
+            locale_tz = unicode(time.tzname[time.localtime()[-1]])
+        ctx['tz'] = locale_tz
+        ctx['changes_url'] = request.childLink("../changes")
+        
+        bn = ctx['builders'] = []
+                
+        for name in builderNames:
+            builder = status.getBuilder(name)
+            top_box = ITopBox(builder).getBox(request)
+            current_box = ICurrentBox(builder).getBox(status)
+            bn.append({'name': name,
+                       'url': request.childLink("../builders/%s" % urllib.quote(name, safe='')), 
+                       'top': top_box.text, 
+                       'top_class': top_box.class_,
+                       'status': current_box.text,
+                       'status_class': current_box.class_,                       
+                        })
 
+        ctx.update(self.phase2(request, changeNames + builderNames, timestamps, eventGrid,
+                  sourceEvents))
 
         def with_args(req, remove_args=[], new_args=[], new_path=None):
             # sigh, nevow makes this sort of manipulation easier
@@ -587,7 +438,7 @@ class WaterfallStatusResource(HtmlResource):
                     newargs[k].append(v)
                 else:
                     newargs[k] = [v]
-            newquery = "&".join(["%s=%s" % (urllib.quote(k), urllib.quote(v))
+            newquery = "&amp;".join(["%s=%s" % (urllib.quote(k), urllib.quote(v))
                                  for k in newargs
                                  for v in newargs[k]
                                  ])
@@ -603,64 +454,18 @@ class WaterfallStatusResource(HtmlResource):
 
         if timestamps:
             bottom = timestamps[-1]
-            nextpage = with_args(request, ["last_time"],
+            ctx['nextpage'] = with_args(request, ["last_time"],
                                  [("last_time", str(int(bottom)))])
-            data += '[<a href="%s">next page</a>]\n' % nextpage
 
-        helpurl = self.path_to_root(request) + "waterfall/help"
-        helppage = with_args(request, new_path=helpurl)
-        data += '[<a href="%s">help</a>]\n' % helppage
+
+        helpurl = path_to_root(request) + "waterfall/help"
+        ctx['help_url'] = with_args(request, new_path=helpurl)
 
         if self.get_reload_time(request) is not None:
-            no_reload_page = with_args(request, remove_args=["reload"])
-            data += '[<a href="%s">Stop Reloading</a>]\n' % no_reload_page
+            ctx['no_reload_page'] = with_args(request, remove_args=["reload"])
 
-        data += "<br />\n"
-	data += self.footer(status, request)
-
-        return data
-
-    def body0(self, request, builders):
-        # build the waterfall display
-        data = ""
-        data += "<h2>Basic display</h2>\n"
-        data += '<p>See <a href="%s">here</a>' % request.childLink("../waterfall")
-        data += " for the waterfall display</p>\n"
-                
-        data += '<table border="0" cellspacing="0">\n'
-        names = map(lambda builder: builder.name, builders)
-
-        # the top row is two blank spaces, then the top-level status boxes
-        data += " <tr>\n"
-        data += td("", colspan=2)
-        for b in builders:
-            text = ""
-            state, builds = b.getState()
-            if state != "offline":
-                text += "%s<br />\n" % state #b.getCurrentBig().text[0]
-            else:
-                text += "OFFLINE<br />\n"
-            data += td(text, align="center")
-
-        # the next row has the column headers: time, changes, builder names
-        data += " <tr>\n"
-        data += td("Time", align="center")
-        data += td("Changes", align="center")
-        for name in names:
-            data += td('<a href="%s">%s</a>' %
-                       (request.childLink("../" + urllib.quote(name)), name),
-                       align="center")
-        data += " </tr>\n"
-
-        # all further rows involve timestamps, commit events, and build events
-        data += " <tr>\n"
-        data += td("04:00", align="bottom")
-        data += td("fred", align="center")
-        for name in names:
-            data += td("stuff", align="center")
-        data += " </tr>\n"
-
-        data += "</table>\n"
+        template = request.site.buildbot_service.templates.get_template("waterfall.html")
+        data = template.render(**ctx)
         return data
     
     def buildGrid(self, request, builders):
@@ -673,13 +478,16 @@ class WaterfallStatusResource(HtmlResource):
         filterCategories = request.args.get('category', [])
         filterBranches = [b for b in request.args.get("branch", []) if b]
         filterBranches = map_branches(filterBranches)
+        filterCommitters = [c for c in request.args.get("committer", []) if c]
         maxTime = int(request.args.get("last_time", [util.now()])[0])
         if "show_time" in request.args:
             minTime = maxTime - int(request.args["show_time"][0])
         elif "first_time" in request.args:
             minTime = int(request.args["first_time"][0])
+        elif filterBranches or filterCommitters:
+            minTime = util.now() - 24 * 60 * 60
         else:
-            minTime = None
+            minTime = 0
         spanLength = 10  # ten-second chunks
         req_events=int(request.args.get("num_events", [self.num_events])[0])
         if self.num_events_max and req_events > self.num_events_max:
@@ -690,8 +498,8 @@ class WaterfallStatusResource(HtmlResource):
         # first step is to walk backwards in time, asking each column
         # (commit, all builders) if they have any events there. Build up the
         # array of events, and stop when we have a reasonable number.
-            
-        commit_source = self.getChangemaster(request)
+
+        commit_source = self.getChangeManager(request)
 
         lastEventTime = util.now()
         sources = [commit_source] + builders
@@ -722,7 +530,9 @@ class WaterfallStatusResource(HtmlResource):
 
         for s in sources:
             gen = insertGaps(s.eventGenerator(filterBranches,
-                                              filterCategories),
+                                              filterCategories,
+                                              filterCommitters,
+                                              minTime),
                              showEvents,
                              lastEventTime)
             sourceGenerators.append(gen)
@@ -767,14 +577,14 @@ class WaterfallStatusResource(HtmlResource):
                         log.msg("pushing", event.getText(), event)
                     events.append(event)
                     starts, finishes = event.getTimes()
-                    firstTimestamp = util.earlier(firstTimestamp, starts)
+                    firstTimestamp = earlier(firstTimestamp, starts)
                     event = get_event_from(sourceGenerators[c])
                 if debug:
                     log.msg("finished span")
 
                 if event:
                     # this is the last pre-span event for this source
-                    lastTimestamp = util.later(lastTimestamp,
+                    lastTimestamp = later(lastTimestamp,
                                                event.getTimes()[0])
                 if debugGather:
                     log.msg(" got %s from %s" % (events, sourceNames[c]))
@@ -807,82 +617,12 @@ class WaterfallStatusResource(HtmlResource):
         assert(len(timestamps) == len(eventGrid))
         return (changeNames, builderNames, timestamps, eventGrid, sourceEvents)
     
-    def phase0(self, request, sourceNames, timestamps, eventGrid):
-        # phase0 rendering
-        if not timestamps:
-            return "no events"
-        data = ""
-        for r in range(0, len(timestamps)):
-            data += "<p>\n"
-            data += "[%s]<br />" % timestamps[r]
-            row = eventGrid[r]
-            assert(len(row) == len(sourceNames))
-            for c in range(0, len(row)):
-                if row[c]:
-                    data += "<b>%s</b><br />\n" % sourceNames[c]
-                    for e in row[c]:
-                        log.msg("Event", r, c, sourceNames[c], e.getText())
-                        lognames = [loog.getName() for loog in e.getLogs()]
-                        data += "%s: %s: %s<br />" % (e.getText(),
-                                                         e.getTimes()[0],
-                                                         lognames)
-                else:
-                    data += "<b>%s</b> [none]<br />\n" % sourceNames[c]
-        return data
-    
-    def phase1(self, request, sourceNames, timestamps, eventGrid,
-               sourceEvents):
-        # phase1 rendering: table, but boxes do not overlap
-        data = ""
-        if not timestamps:
-            return data
-        lastDate = None
-        for r in range(0, len(timestamps)):
-            chunkstrip = eventGrid[r]
-            # chunkstrip is a horizontal strip of event blocks. Each block
-            # is a vertical list of events, all for the same source.
-            assert(len(chunkstrip) == len(sourceNames))
-            maxRows = reduce(lambda x,y: max(x,y),
-                             map(lambda x: len(x), chunkstrip))
-            for i in range(maxRows):
-                data += " <tr>\n";
-                if i == 0:
-                    stuff = []
-                    # add the date at the beginning, and each time it changes
-                    today = time.strftime("<b>%d %b %Y</b>",
-                                          time.localtime(timestamps[r]))
-                    todayday = time.strftime("<b>%a</b>",
-                                             time.localtime(timestamps[r]))
-                    if today != lastDate:
-                        stuff.append(todayday)
-                        stuff.append(today)
-                        lastDate = today
-                    stuff.append(
-                        time.strftime("%H:%M:%S",
-                                      time.localtime(timestamps[r])))
-                    data += td(stuff, valign="bottom", align="center",
-                               rowspan=maxRows, class_="Time")
-                for c in range(0, len(chunkstrip)):
-                    block = chunkstrip[c]
-                    assert(block != None) # should be [] instead
-                    # bottom-justify
-                    offset = maxRows - len(block)
-                    if i < offset:
-                        data += td("")
-                    else:
-                        e = block[i-offset]
-                        box = IBox(e).getBox(request)
-                        box.parms["show_idle"] = 1
-                        data += box.td(valign="top", align="center")
-                data += " </tr>\n"
-        
-        return data
-    
     def phase2(self, request, sourceNames, timestamps, eventGrid,
                sourceEvents):
-        data = ""
+
         if not timestamps:
-            return data
+            return dict(grid=[], gridlen=0)
+        
         # first pass: figure out the height of the chunks, populate grid
         grid = []
         for i in range(1+len(sourceNames)):
@@ -890,7 +630,7 @@ class WaterfallStatusResource(HtmlResource):
         # grid is a list of columns, one for the timestamps, and one per
         # event source. Each column is exactly the same height. Each element
         # of the list is a single <td> box.
-        lastDate = time.strftime("<b>%d %b %Y</b>",
+        lastDate = time.strftime("%d %b %Y",
                                  time.localtime(util.now()))
         for r in range(0, len(timestamps)):
             chunkstrip = eventGrid[r]
@@ -907,9 +647,9 @@ class WaterfallStatusResource(HtmlResource):
                     stuff = []
                     # add the date at the beginning (if it is not the same as
                     # today's date), and each time it changes
-                    todayday = time.strftime("<b>%a</b>",
+                    todayday = time.strftime("%a",
                                              time.localtime(timestamps[r]))
-                    today = time.strftime("<b>%d %b %Y</b>",
+                    today = time.strftime("%d %b %Y",
                                           time.localtime(timestamps[r]))
                     if today != lastDate:
                         stuff.append(todayday)
@@ -993,21 +733,12 @@ class WaterfallStatusResource(HtmlResource):
                             strip[-i+1] = None
                         else:
                             strip[-i].parms['rowspan'] = 1
-        # third pass: render the HTML table
+
+        # convert to dicts
         for i in range(gridlen):
-            data += " <tr>\n";
             for strip in grid:
-                b = strip[i]
-                if b:
-                    # convert data to a unicode string, whacking any non-ASCII characters it might contain
-                    s = b.td()
-                    if isinstance(s, unicode):
-                        s = s.encode("utf-8", "replace")
-                    data += s
-                else:
-                    if noBubble:
-                        data += td([])
-                # Nones are left empty, rowspan should make it all fit
-            data += " </tr>\n"
-        return data
+                if strip[i]:
+                    strip[i] = strip[i].td()
+
+        return dict(grid=grid, gridlen=gridlen, no_bubble=noBubble, time=lastDate)
 
