@@ -1,6 +1,22 @@
+# This file is part of Buildbot.  Buildbot is free software: you can
+# redistribute it and/or modify it under the terms of the GNU General Public
+# License as published by the Free Software Foundation, version 2.
+#
+# This program is distributed in the hope that it will be useful, but WITHOUT
+# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+# details.
+#
+# You should have received a copy of the GNU General Public License along with
+# this program; if not, write to the Free Software Foundation, Inc., 51
+# Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+#
+# Copyright Buildbot Team Members
+
 from buildbot.process.buildstep import LoggingBuildStep, SUCCESS, FAILURE, EXCEPTION
 from buildbot.process.properties import Properties
-from buildbot.scheduler import Triggerable
+from buildbot.schedulers.triggerable import Triggerable
+from twisted.python import log
 from twisted.internet import defer
 
 class Trigger(LoggingBuildStep):
@@ -50,17 +66,23 @@ class Trigger(LoggingBuildStep):
         self.set_properties = set_properties
         self.copy_properties = copy_properties
         self.running = False
+        self.ended = False
         LoggingBuildStep.__init__(self, **kwargs)
         self.addFactoryArguments(schedulerNames=schedulerNames,
                                  updateSourceStamp=updateSourceStamp,
                                  waitForFinish=waitForFinish,
-				                 set_properties=set_properties,
+                                 set_properties=set_properties,
                                  copy_properties=copy_properties)
 
     def interrupt(self, reason):
-        # TODO: this doesn't actually do anything.
         if self.running:
             self.step_status.setText(["interrupted"])
+            return self.end(EXCEPTION)
+
+    def end(self, result):
+        if not self.ended:
+            self.ended = True
+            return self.finished(result)
 
     def start(self):
         properties = self.build.getProperties()
@@ -89,13 +111,11 @@ class Trigger(LoggingBuildStep):
         triggered_schedulers = []
 
         # TODO: don't fire any schedulers if we discover an unknown one
-        dl = []
         for scheduler in self.schedulerNames:
             scheduler = properties.render(scheduler)
             if all_schedulers.has_key(scheduler):
                 sch = all_schedulers[scheduler]
                 if isinstance(sch, Triggerable):
-                    dl.append(sch.trigger(ss, set_props=props_to_set))
                     triggered_schedulers.append(scheduler)
                 else:
                     unknown_schedulers.append(scheduler)
@@ -104,10 +124,13 @@ class Trigger(LoggingBuildStep):
 
         if unknown_schedulers:
             self.step_status.setText(['no scheduler:'] + unknown_schedulers)
-            rc = FAILURE
-        else:
-            rc = SUCCESS
-            self.step_status.setText(['triggered'] + triggered_schedulers)
+            return self.end(FAILURE)
+
+        dl = []
+        for scheduler in triggered_schedulers:
+            sch = all_schedulers[scheduler]
+            dl.append(sch.trigger(ss, set_props=props_to_set))
+        self.step_status.setText(['triggered'] + triggered_schedulers)
 
         if self.waitForFinish:
             d = defer.DeferredList(dl, consumeErrors=1)
@@ -116,16 +139,17 @@ class Trigger(LoggingBuildStep):
 
         def cb(rclist):
             rc = SUCCESS # (this rc is not the same variable as that above)
-            for was_cb, buildsetstatus in rclist:
+            for was_cb, results in rclist:
                 # TODO: make this algo more configurable
                 if not was_cb:
                     rc = EXCEPTION
+                    log.err(results)
                     break
-                if buildsetstatus.getResults() == FAILURE:
+                if results == FAILURE:
                     rc = FAILURE
-            return self.finished(rc)
+            return self.end(rc)
 
         def eb(why):
-            return self.finished(FAILURE)
+            return self.end(FAILURE)
 
         d.addCallbacks(cb, eb)
